import java.awt.*;
import java.awt.image.*;
import java.util.Random;

class Main extends Frame {
    ColorGrid cgrid;
    Label label = new Label();

    Main() {
        super("ColorModel Example");
        ColorModel colorModel = Toolkit.getDefaultToolkit().getColorModel();
        int bitsize = colorModel.getPixelSize();

        if (colorModel instanceof IndexColorModel) {
            // index color model
            cgrid = new ColorGrid(1 << bitsize);
            for (int i=0; i<1<<bitsize; i++) {
                cgrid.setColor(i, colorModel.getRGB(i));
            }
        } else {
            // direct color model
            Random r = new Random();
            cgrid = new ColorGrid(256);
            for (int i=0; i<256; i++) {
                cgrid.setColor(i, r.nextInt());
            }
        }
        add("Center", cgrid);
        add("North", label);
        
        resize(300, 300);
        show();
    }

    public boolean mouseMove(Event evt, int x, int y) {
        ColorModel cm = ColorModel.getRGBdefault();
        int rgb = cgrid.getRGB(x - cgrid.bounds().x, y - cgrid.bounds().y);

        label.setText(cm.getRed(rgb) + " " + cm.getGreen(rgb) 
            + " " + cm.getBlue(rgb) + "    a:" + cm.getAlpha(rgb));
        return true;
    }
        
    public static void main(String[] args) {
        new Main();
    }
}

// This class displays a grid that is used for displaying colors.
class ColorGrid extends Canvas {
    int rows, cols;
    int colors[];
    ColorGrid(int numColors) {
        colors = new int[numColors];
        cols = Math.min(16, numColors);
        rows = (numColors - 1) / cols + 1;
    }

    // Sets the color at the cell located at position 'i'.
    // 'rgb' is a color in the default RGB color model.
    void setColor(int i, int rgb) {
        colors[i] = rgb;
    }

    // Returns the pixel value at (x, y).  The pixel value is encoded
    // using the default color model.
    int getRGB(int x, int y) {
        int cellW = size().width / cols;
        int cellH = size().height / rows;
        
        x /= cellW;
        y /= cellH;

        // Return the last color if out of bounds.
        return colors[Math.min(colors.length-1, y * cols + x)];
    }

    public void paint(Graphics g) {
        int cellW = size().width / cols;
        int cellH = size().height / rows;

        for (int i=0; i<colors.length; i++) {
            int r = i / cols;
            int c = i % cols;

            g.setColor(new Color(colors[i]));
            g.fillRect(c * cellW, r * cellH, cellW, cellH);
        }
    }
}
