import java.awt.*;
import java.awt.image.*;
import java.net.*;
class Main extends Frame {
    Main(String filename) {
        super("Image Example");
        try {
            // Retrieve the image.
            URL url = new URL("file:///" + System.getProperty("user.dir")
               + "/" + filename);

	    add("Center", new ImageCanvas(getToolkit().getImage(url)));
        } catch (Exception e) {
            e.printStackTrace();
        }
        resize(300, 300);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Canvas {
    int scale = 3;
    Image image;
    Image scaledImage;

    ImageCanvas(Image image) {
        this.image = image;
        prepareImage(image, this);
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        int w = image.getWidth(this);
        int h = image.getHeight(this);

        if (w >= 0 || h >= 0) {
            if (g.drawImage(image, 0, 0, this) && scaledImage == null) {
                // The image has been completely loaded.

		// Display any comments.
                if (image.getProperty("comment", this) != Image.UndefinedProperty) {
		    System.out.println(image.getProperty("comment", this));
                }

		// Create a pixel grabber and retrieve the pixels.
		int[] pixels = new int[w * h];
		try {
		    PixelGrabber pg = new PixelGrabber(
			image, 0, 0, w, h, pixels, 0, w);
		    pg.grabPixels();

		    // Check for errors.
		    if ((pg.status() & ImageObserver.ABORT) != 0) {
			System.err.println("Error while fetching image");
			System.exit(1);
		    }
		} catch (Exception e) {
		    e.printStackTrace();
		    System.exit(1);
		}

		// Now replicate the pixels.
		int d = 0, s = 0;
		int[] newpixels = new int[w * h * scale * scale];
		for (int i=0; i<h; i++) 
		    for (int j=0; j<scale; j++) 
			for (int k=0; k<w; k++) 
			    for (int l=0; l<scale; l++) 
				newpixels[d++] = pixels[i*w + k];

		scaledImage = getToolkit().createImage(
		    new MemoryImageSource(w*scale, h*scale,
		    ColorModel.getRGBdefault(), newpixels, 0, w*scale));
	    }
	}
	if (scaledImage != null) {
	    g.drawImage(scaledImage, w, 0, this);
	}
    }
}
