import java.awt.*;
import java.awt.image.*;
class Main extends Frame implements Runnable {
    MediaTracker tracker;
    Image[] images;
    ImageCanvas icv = new ImageCanvas();
    Button btnNext = new Button("Next");
    Button btnPrevious = new Button("Previous");

    Main(String[] files) {
        super("MediaTracker Example");

        tracker = new MediaTracker(icv);
        images = new Image[files.length];
        for (int i=0; i<images.length; i++) {
            images[i] = getToolkit().getImage(files[i]);
            tracker.addImage(images[i], i);
        }

        // First image is progressively rendered.
        if (images.length > 0) {
            icv.setImage(images[0]);
        }

        add("Center", icv);
        add("North", btnPrevious);
        add("South", btnNext);
        resize(460, 250);
        show();
        (new Thread(this)).start();

        // Print overall status information.
        try {
            // We don't use waitForAll() because it forces
            // all images to start loading; we want to load
            // images one at a time.
            // tracker.waitForAll();    
            while (!tracker.checkAll()) {
                Thread.sleep(1000);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        if (tracker.isErrorAny()) {
            System.out.println("Not all images have been successfully loaded.");
            Object[] list = tracker.getErrorsAny();
            for (int i=0; i<list.length; i++) {
                System.out.println(list[i]);
            }
        } else {
            System.out.println("All images have been successfully loaded");
        }
        for (int i=0; i<images.length; i++) {
            int s = tracker.statusID(i, false);

            System.out.print(files[i] + ": ");
            if ((s & MediaTracker.ABORTED) != 0) System.out.print("ABORTED ");
            if ((s & MediaTracker.COMPLETE) != 0) System.out.print("COMPLETE ");
            if ((s & MediaTracker.ERRORED) != 0) System.out.print("ERRORED ");
            if ((s & MediaTracker.LOADING) != 0) System.out.print("LOADING ");
            System.out.println();
        }
    }

    int curImage;
    final int DONE = (MediaTracker.ABORTED | MediaTracker.ERRORED 
                      | MediaTracker.COMPLETE);
    public void paint(Graphics g) {
        if (tracker.isErrorID(curImage)) {
            icv.setImage(null);
        } else {
            icv.setImage(images[curImage]);
        }
        btnNext.enable((tracker.statusID(curImage+1, false)&DONE) != 0);
        btnPrevious.enable((tracker.statusID(curImage-1, false)&DONE) != 0);

        /* See example description for explanation:
        btnNext.enable(tracker.checkID(curImage+1));
        btnPrevious.enable(tracker.checkID(curImage-1));
        */
    }

    public boolean action(Event evt, Object arg) {
        if ("Next".equals(arg)) {
            if (curImage < images.length-1) {
                ++curImage;
                repaint();
            }
            return true;
        } else if ("Previous".equals(arg)) {
            if (curImage > 0) {
                --curImage;
                repaint();
            }
            return true;
        }
        return false;
    }

    public void run() {
        for (int i=0; i<images.length; i++) {
            //tracker.checkID(i);//, true);
            try {
                tracker.waitForID(i);
            } catch (Exception e) {
                e.printStackTrace();
            }
            repaint();
        }
    }

    static public void main(String[] args) {
        new Main(args);
    }
}

class ImageCanvas extends Canvas {
    Image image;
    boolean clear;

    public void setImage(Image image) {
        this.image = image;
        clear = true;
        repaint();
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void paint(Graphics g) {
        if (image == null) {
            g.setColor(Color.red);
            g.fillRect(0, 0, size().width, size().height);
        } else {
            if (clear) {
                g.clearRect(0, 0, size().width, size().height);
                clear = false;
            }
            int w = image.getWidth(this);
            int h = image.getHeight(this);
            if (w >= 0 && h >= 0) {
                g.drawImage(image, (size().width-w)/2, 
                    (size().height-h)/2, this);
            }
        }
    }
}
