import java.net.*;
import java.io.*;
import java.util.Date;

class ClockWorker extends Thread {
    private Socket sock;
    char id;
    public ClockWorker(Socket ss, int i) {
        super();
        sock = ss;
        id = Character.forDigit(i, 10);        // id of worker thread
    }
    public void run() {
        try {
            OutputStream out = sock.getOutputStream();
            String answer = (new Date()).toString();
            byte[] b = new byte[answer.length()];
            answer.getBytes(0, answer.length(), b, 0);
            out.write(id); // first byte indicates worker id
            out.write(b, 0, b.length);
            out.write('\n');
            out.flush();
            sock.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}

class ClockServer extends Thread {
    private ServerSocket srvSock = null;
    public ClockServer(int port, int backlog) {
        super();
        try {
            srvSock = new ServerSocket(port, backlog);
            System.err.println("Server Socket: " + srvSock.toString());
            System.err.println("Socket is connected to: " + 
                               srvSock.getInetAddress());
            System.err.println("Local port: " + srvSock.getLocalPort());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    public void run() {
        if (srvSock == null)
            return;
        Socket sock;
        ClockWorker worker;
        int id = 0;
        while (true) {
            try {
                sock = srvSock.accept();
                worker = new ClockWorker(sock, id++);
                worker.start();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    protected void finalize() {
        if (srvSock != null) {
            try {
                srvSock.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            srvSock = null;
        }
    }
}

class Main {
    static int date_port = 1258;        

    public static void getDate(InetAddress dst, int port) {
        try {
            Socket client = new Socket(dst, port);
            InputStream in = client.getInputStream();

            for (int ch = in.read(); ch > 0; ch = in.read()) {
                System.out.print((char)ch);
            }
            client.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
        // start server
        ClockServer srv = new ClockServer(date_port, 5);
        srv.setDaemon(true);
        srv.start();

        try {
            String msg = "\n";
            InetAddress dst = InetAddress.getLocalHost();

            for (int i = 0; i < 5; i++)
                getDate(dst, date_port);
        } catch (UnknownHostException e) {
                System.err.println("Host not found: " + e);
        }
    }
}

